# TMC Self Managed - Logging setup

[Fluentd](https://docs.fluentd.org) is an open-source data collector for a unified logging layer.
Fluentd allows you to unify data collection and consumption for better use and understanding of data.

## Setup on kubernetes
Fluentd need to be run as a `Deamonset` on kubernetes. Which makes it run on each node where it collects logs from pods
running on that node and sends them to a configured sink. Fluentd can send logs to various services for collection and 
visualization e.g. local file, aws S3 and third party(managed) solutions like logz, kibana etc.
Fluentd can be installed from this [bitnami helm-chart](https://github.com/bitnami/charts/tree/main/bitnami/fluentd/).

We recommend having below config in you fluentd, so that logs from all TMC services are collected. You can configure 
them additionally as per your requirements.

We are using below plugins in the fluentd configuration, which can be installed in fluentd container as below 
```bash
gem update --ignore-dependencies fluent-plugin-kubernetes_metadata_filter
gem install fluent-plugin-parser-cri --no-document
```
_These commands can be passed as bash script to the bitnami helm-chart via `forwarder.initScripts` flag._

```nginx configuration
# Specify the source of logs to be collected.
# Custom logs parser can be provided via environment variable `FLUENT_CONTAINER_TAIL_PARSER_TYPE` to the container
# options are documented here - https://docs.fluentd.org/configuration/parse-section
# By default we are using `cri` parser - https://github.com/fluent/fluent-plugin-parser-cri 
<source>
  @type tail
  @id in_tail_container_logs
  path /var/log/containers/*.log
  pos_file /var/log/fluentd-containers.log.pos
  tag kubernetes.*
  read_from_head true
  <parse>
    @type "#{ENV['FLUENT_CONTAINER_TAIL_PARSER_TYPE'] || 'cri'}"
  </parse>
</source>

# kuberntes metadata plugin, this adds k8s metadata info to logs(eg. namspace, pods, labels etc.)
# https://github.com/fabric8io/fluent-plugin-kubernetes_metadata_filter
<filter kubernetes.**>
  @type kubernetes_metadata
  @id filter_kube_metadata
</filter>

# Drop logs from fluentd pod itself (optional).
<match kubernetes.**fluentd**.log>
  @type null
</match>

# Parse logs to json format
# Read about these values at https://docs.fluentd.org/filter/parser
<filter kubernetes.var.log.containers.**>
  @type parser
  <parse>
    @type json
  </parse>
  key_name message
  emit_invalid_record_to_error false
  reserve_data true
  reserve_time true
</filter>

# Remove unwated keys
<filter kubernetes.var.log.containers.**tmc-local**.log>
  @type record_modifier
  remove_keys $.intent_obj.spec_body.metadata.managedFields
</filter>
```

## Disable logs collection for TMC Self managed
If for some reason you don't want to collect logs from TMC, you can put below config in your fluentd.
```nginx configuration
# drop all logs from tmc-local namespace
<match kubernetes.var.log.**tmc-local**.log>
  @type null
</match>

# drop all logs from a specific service. 
<match kubernetes.var.log.**service-name**.log>
  @type null
</match>
```

## Export fluentd metrics for prometheus.

You can also setup monitoring for fluentd pods, by exposing metrics for prometheus on a port.
```nginx configuration
# Documentation here - https://docs.fluentd.org/monitoring-fluentd/monitoring-prometheus
<filter kubernetes.**>
  @type prometheus
  <metric>
    name fluentd_input_status_num_records_total
    type counter
    desc The total number of incoming records
    <labels>
      tag ${tag}
      hostname ${hostname}
    </labels>
  </metric>
</filter>

# Prometheus metric exposed on 0.0.0.0:24231/metrics
<source>
  @type prometheus
  bind "#{ENV['FLUENTD_PROMETHEUS_BIND'] || '0.0.0.0'}"
  port "#{ENV['FLUENTD_PROMETHEUS_PORT'] || '24231'}"
  metrics_path "#{ENV['FLUENTD_PROMETHEUS_PATH'] || '/metrics'}"
</source>

<source>
  @type prometheus_output_monitor
  interval 10
  <labels>
    hostname ${hostname}
  </labels>
</source>
```